
;; Atmel Pilot - Copyright 2008 by Michael Kohn
;; Email: mike@mikekohn.net
;;   Web: http://www.mikekohn.net/
;;
;; Control RC plane servos or DC motors with PWM

.include "m168def.inc"

;  cycles  time   @20MHz:
;   20000: 1ms
;   10000: 0.5ms
;    1000: 0.05ms

; interrupts  time   @0.05ms increments
;         20  1ms 
;         40  2ms 
;        400 20ms 

; r0  = 0
; r2  = PORTB on servo reset (20ms of interrupts)
; r3  = high byte of interrupt count
; r4  = 16
; r5  = 20
; r6  = 24
; r17 = temp
; r18 = temp
; r19 = temp
; r21 = temp
; r23 = low byte of interrupt count
; r20 = UART in register
;

; Command byte:
; 7 6 5 4 3 2 1 0
; V V V V V M M M
;
; M = motor number
; V = value from 0 to 20; 30 = set as servo ; 31 = set as DC motor

; note: CLKSEL 0110

; max count for signal high (little endian)
.equ M0_MAX = SRAM_START
.equ M1_MAX = SRAM_START+2
.equ M2_MAX = SRAM_START+4
.equ M3_MAX = SRAM_START+6
.equ M4_MAX = SRAM_START+8
.equ M5_MAX = SRAM_START+10
.equ M6_MAX = SRAM_START+12
.equ M7_MAX = SRAM_START+14

; motor configuration (0 = servo  1 = dc motor)
.equ M0_MODE = SRAM_START+16
.equ M1_MODE = SRAM_START+17
.equ M2_MODE = SRAM_START+18
.equ M3_MODE = SRAM_START+19
.equ M4_MODE = SRAM_START+20
.equ M5_MODE = SRAM_START+21
.equ M6_MODE = SRAM_START+22
.equ M7_MODE = SRAM_START+23

; update count for signal high (little endian)
; during interrupt these values get written to Mx_MAX
.equ M0_UPDATE = SRAM_START+24
.equ M1_UPDATE = SRAM_START+26
.equ M2_UPDATE = SRAM_START+28
.equ M3_UPDATE = SRAM_START+30
.equ M4_UPDATE = SRAM_START+32
.equ M5_UPDATE = SRAM_START+34
.equ M6_UPDATE = SRAM_START+36
.equ M7_UPDATE = SRAM_START+38

.cseg

.org 0x000
  rjmp start
.org 0x016
  rjmp service_motors
.org 0x01a
  rjmp service_motors

;; FIXME - erase this padding.. it's dumb
.org 0x020

start:
  ;; I'm busy.  Don't interrupt me!
  cli

  ;; clear the memory where servo info is
  eor r0, r0   ; r0 = constant 0
  ldi r17, 40
  ldi r31, M0_MAX>>8 
  ldi r30, M0_MAX&255
memset_servos:
  st Z+, r0
  dec r17
  brne memset_servos

  ;; r4 = 16, r5 = 20, r6 = 24
  ldi r17, 16
  mov r4, r17
  ldi r17, 20
  mov r5, r17
  ldi r17, 24
  mov r6, r17

  ;; r3:r23 = interrupt counts
  eor r3, r3
  eor r23, r23

  ;; put input state (r21) in command mode
  ;; ser r21

  ;; Set up stack ptr
  ldi r17, RAMEND>>8
  out SPH, r17
  ldi r17, RAMEND&255
  out SPL, r17

  ;; Set up PORTB
  ser r17
  ldi r18, 0
  out DDRB, r17             ; entire port B is output
  out PORTB, r18            ; turn off all of port B for fun
  ;out DDRC, r17             ; entire port C is output
  ;out PORTC, r18            ; turn off all of port C for fun

  ;; Set up rs232 baud rate
  ;eor r17, r17
  sts UBRR0H, r0
  ldi r17, 129
  sts UBRR0L, r17           ; 129 @ 20MHz = 9600 baud

  ;; Set up rs232 options
  ldi r17, (1<<UCSZ00)|(1<<UCSZ01)    ; sets up data as 8N1
  sts UCSR0C, r17
  ldi r17, (1<<TXEN0)|(1<<RXEN0)      ; enables send/receive
  sts UCSR0B, r17
  eor r17, r17
  sts UCSR0A, r17

  ;; Set up TIMER1
  lds r17, PRR
  andi r17, 255 ^ (1<<PRTIM1)
  sts PRR, r17                   ; turn of power management bit on TIM1

  ldi r17, (1000>>8)
  sts OCR1AH, r17
  ldi r17, (1000&0xff)            ; compare to 1000 clocks (0.05ms)
  sts OCR1AL, r17

  ldi r17, (1<<OCIE1A)
  sts TIMSK1, r17                ; enable interrupt comare A 
  eor r17, r17
  sts TCCR1C, r17
  ; r17 = 0 still
  sts TCCR1A, r17                ; normal counting (0xffff is top, count up)
  ldi r17, (1<<CS10)|(1<<WGM12) ; CTC OCR1A
  sts TCCR1B, r17                ; prescale = 1 from clock source
  
  ; Fine, I can be interrupted now
  sei

main:
  lds r20, UCSR0A         ; poll uart to see if there is a data waiting
  sbrs r20, RXC0
  rjmp main               ; if no data, loop around

  lds r20, UDR0

  ;; ECHO - just for checking things are okay
  mov r24, r20
  call send_char

  ; cpi r21, 255
  ; brne process_command
  mov r21, r20
  andi r21, 7                    ; r21 now holds motor number
  lsr r20
  lsr r20
  lsr r20                        ; r20 now holds value

  cpi r20, 31
  breq config
  cpi r20, 30
  breq config

  ;; rjmp main

process_command:
  ;sbrc r21, 6
  ;rjmp config
  ;subi r21, '0'                  ; r21: servo number
  ;subi r20, 'A'                  ; r20: value
  ;andi r21, 7                    ; servo number & 7  (0 to 7)
  ;andi r20, 31                   ; should be between 0 and 20 actually

  ; read config to to see if it's a DC motor or servo
  mov r28, r21                   ; this is kinda wrong but okay for the
  add r28, r4                    ; r28 = r21 + 16
  ldi r29, (M0_UPDATE>>8)        ; FIXME - move this up later
  ld r16, Y
  sbrc r16, 0
  rjmp dc_motor

  lsl r21                        ; r21 = r21 * 2
  mov r28, r21
  add r28, r6                    ; r28 = r21 + 24
  add r20, r5                    ; add r20 + 20  (value of servo is 20 to 40)
  st Y+, r20
  st Y+, r0
  ser r21
  rjmp main

dc_motor:
  lsl r21                        ; r21 = r21 * 2
  mov r28, r21
  add r28, r6                    ; r28 = r21 + 24
  ldi r31, (dc_motor_lookup*2)>>8
  ldi r30, (dc_motor_lookup*2)&255
  lsl r20                        ; r20 = r20 * 2
  add r30, r20                   ; Z = dc_motor_lookup + (r20 * 2)
  brcc no_carry
  inc r31
no_carry:
  lpm r16, Z+                    ; *Y = *Z
  st Y+, r16
  lpm r16, Z
  st Y, r16

  ser r21
  rjmp main

config:
  subi r20, 30                   ; Value (30 or 31) - 30

  mov r28, r21                   ; this is kinda wrong but okay for atmega168
  add r28, r4                    ; r28 = r28 + 16
  ldi r29, (M0_MODE>>8)
  st Y, r20                      ; store value in servo config

  rjmp main


service_motors:
  ; increment interrupt counter
  inc r23
  brne r23_not_0
  inc r3          ; r3:r23 ++

r23_not_0:
  ; 400 (0x0190) interrupts = 20ms
  sbrs r3, 0
  rjmp not_20ms
  cpi r23, 0x90
  brne not_20ms   ; if (r3:r23 != 0x0190) goto not_20ms

  eor r3, r3
  eor r23, r23    ; reset interrupt counter to 0
  eor r2, r2      ; all servos on
  dec r2          ; r2 = 0 - 1 = 255

not_20ms:

  ldi r17, 1
  ldi r27, M0_MAX>>8 
  ldi r26, M0_MAX&255     ; X = M0_MAX
servo_loop:
  ld r18, X+              ; r19:r18 = *(M0_MAX+count)
  ld r19, X+
  cp r18, r23
  brne servo_loop_next
  cp r19, r3
  brne servo_loop_next    ; if (r19:r18 != r3:r23) goto servo_loop_next

  ser r18
  eor r18, r17            ; mask out motor so it turns off
  and r2, r18             ; r2 = (255 ^ r17) & r2

servo_loop_next:
  lsl r17                 ; r17 = r17 << 1
  brne servo_loop         ; next

  out PORTB, r2           ; turn on servos if they should be on

  ; 399 (0x018A) interrupts: update all servo MAX values
  sbrs r3, 0
  reti
  cpi r23, 0x8a
  breq update   ; if (r3:r23 != 0x018A) goto no_update
  reti

update:
  ;; Unrolled loop.  I have a feeling I should have unrolled some of
  ;; the loops above, maybe with python code to generate it.  Code might
  ;; be harder to read, but it would gain some speed and free up a couple
  ;; registers.
  lds r17, M0_UPDATE
  lds r18, M0_UPDATE+1
  sts M0_MAX, r17
  sts M0_MAX+1, r18

  lds r17, M1_UPDATE
  lds r18, M1_UPDATE+1
  sts M1_MAX, r17
  sts M1_MAX+1, r18

  lds r17, M2_UPDATE
  lds r18, M2_UPDATE+1
  sts M2_MAX, r17
  sts M2_MAX+1, r18

  lds r17, M3_UPDATE
  lds r18, M3_UPDATE+1
  sts M3_MAX, r17
  sts M3_MAX+1, r18

  lds r17, M4_UPDATE
  lds r18, M4_UPDATE+1
  sts M4_MAX, r17
  sts M4_MAX+1, r18

  lds r17, M5_UPDATE
  lds r18, M5_UPDATE+1
  sts M5_MAX, r17
  sts M5_MAX+1, r18

  lds r17, M6_UPDATE
  lds r18, M6_UPDATE+1
  sts M6_MAX, r17
  sts M6_MAX+1, r18

  lds r17, M7_UPDATE
  lds r18, M7_UPDATE+1
  sts M7_MAX, r17
  sts M7_MAX+1, r18
  reti

; void send_char(r24)  : r14 trashed
send_char:
  lds r14, UCSR0A     ; check to see if it's okay to send a char
  sbrs r14, UDRE0
  rjmp send_char      ; if it's not okay, loop around :)
  sts UDR0, r24       ; output a char over rs232
  ret


signature:
.db "Atmel Pilot - Copyright 2008 - Michael Kohn - Version 0.03"

dc_motor_lookup:
.db 0, 0, 20, 0, 40, 0, 60, 0, 80, 0, 100, 0, 120, 0, 140, 0, 160, 0 
.db 180, 0, 200, 0, 220, 0, 240, 0, 4, 1, 24, 1, 44, 1, 64, 1, 84, 1
.db 104, 1, 124, 1, 144, 1, 144, 1, 144, 1, 144, 1


