#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <termios.h>
#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>

static int fd=0;
static struct termios oldtio,newtio;

/*************************************************************

naken_plugin_init() - This function will get called when
Naken Web starts up.  You can set up whatever you need to
in here including any threads

*************************************************************/
void naken_plugin_init()
{
  printf("naken_plugin_init() called\n");

  fd=open("/dev/tts/1",O_RDWR,O_NOCTTY);
  if (fd==-1)
  {
    printf("Couldn't open serial device.\n");
    fd=0;
    return;
  }
    else
  {
    printf("Serial port opened successfully.\n");
  }

  tcgetattr(fd,&oldtio);

  memset(&newtio,0,sizeof(struct termios));
  newtio.c_cflag=B9600|CS8|CLOCAL|CREAD;
  newtio.c_iflag=IGNPAR;
  newtio.c_oflag=0;
  newtio.c_lflag=0;
  newtio.c_cc[VTIME]=0;
  newtio.c_cc[VMIN]=1;

  tcflush(fd, TCIFLUSH);
  tcsetattr(fd,TCSANOW,&newtio);

}

/*************************************************************

naken_plugin_get() - This function is called when a user
requests your plugin.  Querystring holds everything in the
URL after the ? null terminated so for example if the
user typed:

http://localhost/test_plugin.cgi?user_id=1&name=mike%20kohn

then querystring="user_id=1&name=mike%20kohn";

fd_out is used to output html or whatever through Naken Web
to the user's screen.

The return value should be 0 for a keep-alive connection
and there is no problem (content-length must be sent then)
or -1 to tell Naken Web to close the connection.

*************************************************************/

int naken_plugin_get(int fd_out, char *querystring)
{
//char buffer[]={"  \r\n"};
char buffer[]={ "Content-Type: text/html\r\nContent-Length: 4\r\n\r\nXX\r\n" };
unsigned char command;
int ptr,r;

  ptr=sizeof(buffer)-5;
  printf("naken_plugin_get() called %s\n",querystring);

  buffer[ptr]=querystring[0];
  buffer[ptr+1]=querystring[1];

  send(fd_out,buffer,sizeof(buffer)-1,0);

  if (fd==0) return -1;

  if (querystring[0]>='0' && querystring[0]<='7')
  {
    command=querystring[0]-'0';

    r=querystring[1]-'A';
    if (r>=0 && r<30)
    {
      command=(r<<3)|command;
    }
      else
    {
      printf("Illegal motor value\n");
      return 0;
    }
  }
    else
  if (querystring[0]>='A' && querystring[0]<='H')
  {
    command=querystring[0]-'A';

    if (querystring[1]=='A')
    { command=(30<<3)|command; }
      else
    if (querystring[1]=='B')
    { command=(31<<3)|command; }
      else
    {
      printf("Illegal motor config\n");
      return 0;
    }
  }
    else
  {
    printf("Illegal motor\n");
    return 0;
  }

  printf("send=%02x\n",command);

  write(fd,&command,1);
  command=0;
  read(fd,&command,1);

  printf("reply=%02x\n",command);

  return 0;
}

/*************************************************************

naken_plugin_get() - This function is called when a user
submits a form using the post method.  Querystring here is
the same as it is for get.  Naken Web expects this function
to read content_length bytes from fd_in (which will be the
submitted data).  Otherwise it's identical to naken_plugin_get().

*************************************************************/

int naken_plugin_post(int fd_out, char *querystring, int fd_in, int content_length)
{
char buf[1024];
int c,n;

  printf("naken_plugin_post() called %s\n",querystring);

  c=0;
  while(c<content_length)
  {
    if (content_length-c<1023)
    { n=recv(fd_in, buf, content_length, 0); }
      else
    { n=recv(fd_in, buf, 1023, 0); }

    if (n<1) break;
    buf[n]=0;
    printf("recv'd %s\n",buf);
    c=c+n;
  }

  /* This is the wrong way to use send().. don't copy this */
  send(fd_out,"Content-Type: text/html\r\n\r\n",sizeof("Content-Type: text/html\r\n\r\n")-1,0);
  send(fd_out,"Post Querystring is ",sizeof("Get Querystring is ")-1,0);
  send(fd_out,querystring,strlen(querystring),0);

  return -1;
}



