#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <linux/joystick.h>

#include "ptz_network.h"

/*

    1            3
        linksys
         front
    0            2
*/

int main(int argc, char *argv[])
{
struct js_event e;
struct website_info_t website_info;
char *joy_dev=0;
char command[128];
int fd;
int r;
fd_set readset;
struct timeval tv;
int dirty=0;
int thrust=0,roll=0,pitch=0;
int new_thrust=0,new_roll=0,new_pitch=0;
int motor0=0,motor1=0,motor2=0,motor3=0;

  if (argc<2)
  {
    printf("Usage: joystick [options] <website address>\n");
    printf("   -p <port>\n");
    printf("   -u <username:password>\n");
    printf("   -j <joystick device: /dev/input/js0>\n\n");
    exit(0);
  }

  memset(&website_info,0,sizeof(struct website_info_t));
  website_info.port=80;

  /* Read in command line arguments */

  for (r=1; r<argc; r++)
  {
    if (strcmp(argv[r],"-p")==0)
    { website_info.port=atoi(argv[++r]); }
      else
    if (strcmp(argv[r],"-u")==0)
    { website_info.username_password=base64_encode(argv[++r]); }
      else
    if (strcmp(argv[r],"-j")==0)
    { joy_dev=argv[++r]; }
      else
    {
      website_info.host=argv[r];
    }
  }

  /* Open joystick /dev */

  if (joy_dev==0)
  { fd=open("/dev/input/js0",O_RDONLY); }
    else
  { fd=open(joy_dev,O_RDONLY); }

  if (fd==0)
  {
    printf("Could no open joystick device\n");
    exit(1);
  }

  /* Setup non-blocking I/O */

  fcntl(fd,F_SETFL,O_NONBLOCK);

  init_socket();

  // set up all motors as DC motors
  send_command(&website_info,"/pilot.plg?AB");
  send_command(&website_info,"/pilot.plg?BB");
  send_command(&website_info,"/pilot.plg?CB");
  send_command(&website_info,"/pilot.plg?DB");

  /* Main loop for reading the joystick device.  Use non-blocking I/O
     to ignore many joystick changes in a row. */

  while(1)
  {
    tv.tv_sec=10;
    tv.tv_usec=0;

    FD_ZERO(&readset);
    FD_SET(fd,&readset);

    if ((r=select(fd+1,&readset,NULL,NULL,&tv))==-1)
    {
      printf("Select error\n");
      break;
    }

    if (r==0) continue;

    /* read joystick events and wait for things to settle down 
       before actually sending PTZ commands.  dx,dy,dz are the
       current changed values of the joystick.  After there are
       no more joystick events queued up, send events to the camera
       based on if they are not equal to the current x,y,z. */

    while(1)
    {
      r=read(fd, &e, sizeof(struct js_event));
      if (r==-1) break;

#ifdef DEBUG
printf("number=%d value=%d\n",e.number,e.value);
#endif

      if (e.type==JS_EVENT_AXIS)
      {
        if (e.number==0)  /* Roll */
        {
           if (e.value<-4000) // tilted left
           {
             new_roll=(e.value+4000)*5/28767;
           }
             else
           if (e.value>4000)  // tilted right
           {
             new_roll=(e.value-4000)*5/28767;
           }
             else
           {
             new_roll=0;
           }
        }
          else
        if (e.number==1)  /* Pitch */
        {
           if (e.value<-4000) // tilted forward
           {
             new_pitch=(e.value+4000)*5/28767;
           }
             else
           if (e.value>4000) // tilted back
           {
             new_pitch=(e.value-4000)*5/28767;
           }
             else
           {
             new_pitch=0;
           }
        }
          else
        if (e.number==3)  /* thrust */
        {
          new_thrust=e.value;
          if (new_thrust>0) new_thrust=0;
          new_thrust=-new_thrust;
          new_thrust=new_thrust*20/32767;
        }
      }

      if (thrust!=new_thrust ||
          pitch!=new_pitch ||
          roll!=new_roll)
      {
        roll=new_roll;
        pitch=new_pitch;
        thrust=new_thrust;

        motor0=thrust;
        motor1=thrust;
        motor2=thrust;
        motor3=thrust;

        if (roll>0)
        {
          motor2=motor2-roll;
          motor3=motor3-roll;
        }
          else
        if (roll<0)
        {
          motor0=motor0+roll;
          motor1=motor1+roll;
        }

        if (pitch>0)
        {
          motor1=motor1-pitch;
          motor3=motor3-pitch;
        }
          else
        if (pitch<0)
        {
          motor0=motor0+pitch;
          motor2=motor2+pitch;
        }

        if (motor0<0) motor0=0;
        if (motor1<0) motor1=0;
        if (motor2<0) motor2=0;
        if (motor3<0) motor3=0;

        if (motor0>20) motor0=20;
        if (motor1>20) motor1=20;
        if (motor2>20) motor2=20;
        if (motor3>20) motor3=20;

        dirty=1;
      }
    }


#ifdef DEBUG
/*
    printf("  time: %d\n",e.time);
    printf("Event-----------\n");
    printf("  time: %d\n",e.time);
    printf(" value: %d\n",e.value);
    printf("  type: %d\n",e.type);
    printf("number: %d\n",e.number);
*/
#endif

    /* Make a web connection to the camera to move it.  These URLs can
       be changed for other brand of cameras.  The following code works
       with Axis cameras. */

    if (dirty!=0)
    {
      sprintf(command,"/pilot.plg?0%c",motor0+'A');
      send_command(&website_info,command);
      sprintf(command,"/pilot.plg?1%c",motor1+'A');
      send_command(&website_info,command);
      sprintf(command,"/pilot.plg?2%c",motor2+'A');
      send_command(&website_info,command);
      sprintf(command,"/pilot.plg?3%c",motor3+'A');
      send_command(&website_info,command);

      dirty=0;
#ifdef DEBUG
printf("Command: %s\n",command);
#endif
    }

  }

  close_socket(&website_info);
  close(fd);

  return 0; 
}

